

from dataclasses import dataclass
from enum import Enum
from typing import Any, List, Optional

from bs4 import ResultSet, SoupStrainer, Tag


def is_abbr(tag):
    return tag.name == 'abbr'


@dataclass()
class Definition:
    types: [str]
    definition: str
    examples: Optional[List[str]]
    def __init__(self, soup: Tag):
        ordinal = soup.find('span', {'class': 'n_acep'})
        types = ordinal.find_next_siblings('abbr', {'class': ['d', 'g']})
        ordinal.decompose()
        self.types = [t.text for t in types]
        for t in types:
            t.decompose()
        examples = soup.find_all('span', {'class': 'h'})
        self.examples = [sp.text for sp in examples]
        for ex in examples:
            ex.decompose()
        self.definition = soup.text.strip()

@dataclass()
class DefinitionX:
    types: [str]
    definition: str
    examples: Optional[List[str]]
    synonyms: Optional[List[str]]
    antonyms: Optional[List[str]]
    def __init__(self, soup: Tag):
        self.synonyms = []
        self.antonyms = []
        ordinal = soup.find('span', {'class': 'n_acep'})
        types = ordinal.find_next_siblings('abbr', {'class': ['d', 'g']})
        ordinal.decompose()
        self.types = [t.text for t in types]
        for t in types:
            t.decompose()
        examples = soup.find_all('span', {'class': 'h'})
        self.examples = [sp.text for sp in examples]
        for ex in examples:
            ex.decompose()
        table = soup.find('table', {'class': 'sinonimos'})
        if table is not None:
            rows = table.find_all("tr")
            self.synonyms = [s.text for s in rows[0].find_all("td")[1].find_all("span", {"class": 'sin'})]
            if (len(rows)) == 2:
                self.antonyms = [s.text for s in rows[1].find_all("td")[1].find_all("span", {"class": 'sin'})]
            if table:
                table.decompose()
        self.definition = soup.text.strip()


@dataclass()
class Phrase:
    text: str
    definitions: [Definition]


@dataclass()
class Word:
    text: str
    etymology: str
    definitions: [DefinitionX]
    phrases: Optional[List[Phrase]]

    def __init__(self, soup: Tag):
        header = soup.find('header')
        sub = header.find("sup")
        if sub:
            sub.decompose()
        self.text = header.text
        ety = soup.find('p', {'class': 'n2'})
        if ety:
            self.etymology = ety.text
        else:
            ety = ""
        self.definitions = [DefinitionX(sp) for sp in soup.find_all('p', {'class': 'j'})]
        phrases: List[Tag] = [ph for ph in soup.find_all('p', {'class': ['k5', 'k6', 'm']})]
        defs = []
        phr = None
        self.phrases = []
        for ph in phrases:
            if ph.attrs['class'] == ['k5'] or ph.attrs['class'] == ['k6']:
                if phr and defs:
                    self.phrases.append(Phrase(phr, defs))
                phr = ph.text.strip()
                defs = []
                continue
            if ph.attrs['class'] == ['m']:
                defs.append(Definition(ph))
                continue



import requests
from bs4 import BeautifulSoup
from .model import Definition, Phrase, Word

BASE_URL = "https://dle.rae.es"
session = requests.session()
session.headers.update({'User-Agent': 'Mozilla/5.0'})


def by_words(word: str):
    resp = session.get(f"{BASE_URL}/{word}?m=form")
    soup = BeautifulSoup(resp.text, 'html.parser')
    entries = soup.select('#resultados > article')
    if entries:
        words = []
        for entry in entries:
            try:
                wd = Word(entry)
                words.append(wd)
            except:
                header = entry.find('header')
                linked_entries = header.find_next_siblings('p', {'class': ['l', 'l3', 'b']})
                if linked_entries:
                    # Search for Allen, found "llave Allen"
                    for linked in linked_entries:
                        path = linked.find('a').attrs['href']
                        id = path.split('#')[1]
                        url = f"{BASE_URL}{path}"
                        resp = session.get(url)
                        soup = BeautifulSoup(resp.text, 'html.parser')
                        entries = soup.find_all('p', {'id': id})
                        if entries:
                            if entries[0].attrs['class'] == ['k5'] or entries[0].attrs['class'] == ['k6']:
                                # Entry is a phrase
                                ph = entries[0].text
                                defi = Definition(entries[0].find_next_sibling('p', {'class': 'm'}))
                                # TODO: support multiple defs
                                wd = Phrase(ph, [defi])
                            else:
                                wd = Word(entry[0])
                            words.append(wd)

        return words
    else:
        others = soup.select('#resultados > .otras > .n1 > a')
        if others:
            words = []
            for o in others:
                path = o.attrs['href']
                id = path.split('#')[1]
                url = f"{BASE_URL}{path}"
                resp = session.get(url)
                soup = BeautifulSoup(resp.text, 'html.parser')
                entry = soup.find_all("article", {"id": id})
                wd = Word(entry[0])
                words.append(wd)
        else:
            # Search for fallecio, found fallecer and fallecido
            words = []
            links = soup.select('#resultados > .item-list > .n1 > a')
            for link in links:
                path = link.attrs['href']
                url = f"{BASE_URL}{path}"
                resp = session.get(url)
                soup = BeautifulSoup(resp.text, 'html.parser')
                entry = soup.select(f'article')
                wd = Word(entry[0])
                words.append(wd)
        return words


def by_expressions(expr: str):
    resp = session.get(f"{BASE_URL}/{expr}?m=form2")
    soup = BeautifulSoup(resp.text, 'html.parser')
    entries = soup.select('#resultados > article')
    words = []
    for entry in entries:
        wd = Word(entry)
        words.append(wd)
    return words

def exact_match():
    pass

def starts_with():
    pass

def ends_with():
    pass

def contains():
    pass

def anagrams():
    pass

def random():
    pass



